part of 'create_role_bloc.dart';

@immutable
sealed class CreateRoleEvent {}

class CreateRoleInitEvent extends CreateRoleEvent {
  final PositionEntity role;

  CreateRoleInitEvent(this.role);
}

class CreateRoleSelectSkillEvent extends CreateRoleEvent {
  final BusinessSkillModel skill;

  CreateRoleSelectSkillEvent(this.skill);
}

class ExpandRoleEvent extends CreateRoleEvent {
  ExpandRoleEvent();
}

class SetRoleStartTimeEvent extends CreateRoleEvent {
  final DateTime startTime;

  SetRoleStartTimeEvent(this.startTime);
}

class SetRoleEndTimeEvent extends CreateRoleEvent {
  final DateTime endTime;

  SetRoleEndTimeEvent(this.endTime);
}

class CreateRoleSelectDepartmentEvent extends CreateRoleEvent {
  final DepartmentModel item;

  CreateRoleSelectDepartmentEvent(this.item);
}

class CreateRoleSelectCountEvent extends CreateRoleEvent {
  final int item;

  CreateRoleSelectCountEvent(this.item);
}

class CreateRoleSelectBreak extends CreateRoleEvent {
  final int item;

  CreateRoleSelectBreak(this.item);
}

class CreateRoleSetScheduleEvent extends CreateRoleEvent {
  final List<RoleScheduleEntity> schedule;

  CreateRoleSetScheduleEvent(this.schedule);
}

class ValidationPositionFailedEvent extends CreateRoleEvent {
  final PositionValidationState validationState;

  ValidationPositionFailedEvent(this.validationState);
}
